/* misc.h - miscellaneous
 *      Copyright (C) 2002 Klarlvdalens Datakonsult AB
 *
 * This file is part of DirMngr.
 *
 * DirMngr is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * DirMngr is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifndef MISC_H
#define MISC_H

#include <ksba.h>

#include "error.h"

/* A type to hold the ISO time.  Note that this this is the same as
   the the KSBA type ksba_isotime_t. */
typedef char dirmngr_isotime_t[16];


/* Return the current time in ISO format. */
void get_isotime (dirmngr_isotime_t timebuf);


/* Copy one ISO date to another, this is inline so that we can do a
   sanity check. */
static inline void
copy_time (dirmngr_isotime_t d, const dirmngr_isotime_t s)
{
  if (*s && (strlen (s) != 15 || s[8] != 'T'))
    BUG ();
  strcpy (d, s);
}

/* Convert hex encoded string back to binary. */
size_t unhexify (unsigned char *result, const char *string);

/* Returns SHA1 hash of the data. */
char* hashify_data( const char* data, size_t len );

/* Returns data as a hex string. */
char* hexify_data( const unsigned char* data, size_t len );

/* Returns the serial number as a hex string.  */
char* serial_hex ( ksba_sexp_t serial );

/* Take an S-Expression encoded blob and return a pointer to the
   actual data as well as its length. */
const unsigned char *serial_to_buffer (const ksba_sexp_t serial,
                                       size_t *length);


#endif /* MISC_H */
